# coding: utf-8
from __future__ import unicode_literals, print_function
import sys
import json
from collections import OrderedDict
from pprint import pformat
from mako.template import Template
from io import open
from datetime import datetime, date


def _defaultSerialize(obj):
    if isinstance(obj, (datetime, date)):
        return obj.isoformat()
    raise TypeError("Type %s not serializable" % type(obj))


def toJson(excel, output, encoding="utf-8"):
    _dict = OrderedDict()
    for sheet in excel:
        _dict[sheet.name] = list(sheet)

    if sys.version_info[0] == 2:
        with open(output, mode="wb") as f:
            json.dump(
                _dict,
                f,
                ensure_ascii=False,
                encoding=encoding,
                default=_defaultSerialize,
            )
        return

    with open(output, mode="w", encoding=encoding) as f:
        json.dump(_dict, f, ensure_ascii=False, indent=4, default=_defaultSerialize)
    pass


def toMako(excel, output, template, encoding="utf-8"):
    with open(output, mode="w", encoding=encoding) as ouputfile:
        text = ""
        with open(template, mode="r", encoding=encoding) as f:
            text = Template(f.read()).render(excel=excel, format=pformat)
        ouputfile.write(text)
    pass


def toMsgPack(excel, output, encoding="utf-8"):
    import msgpack

    _dict = OrderedDict()
    for sheet in excel:
        _dict[sheet.name] = list(sheet)

    with open(output, mode="wb") as f:
        f.write(msgpack.packb(_dict, default=_defaultSerialize))
    pass



import os
from conftest import testdata_dir
from excel2xx import validator

validator_dir = testdata_dir + "/" + "_validator"


def gen_cases():
    os.makedirs(validator_dir, exist_ok=True)
    """
    """
    pass


def test_validator():
    gen_cases()
    validator.run({}, testdata_dir)
    assert 1 == 1



import os
from excel2xx import Excel

root_dir = os.path.dirname(os.path.dirname(__file__))


def test_fields():
    fpath = root_dir + "/example/test.xlsx"
    l = Excel(fpath)[0].toList()
    assert l
    assert l[0]["fieldJson"] == {"a": 1, "b": 2}
    assert l[0]["fieldJson"] != {"a": 1, "b": 2, "c": 3}
    d = Excel(fpath)[0].toDict()
    assert d



# encoding=utf-8
from collections import OrderedDict
from excel2xx.fieldmeta import FieldMeta


def toKV(sheet, fields=("key", "type", "value")):
    d = OrderedDict()
    for row in sheet:
        key = row[fields[0]]
        valType = row[fields[1]]
        val = row[fields[2]]
        if isinstance(val, float):
            val = intIf(val)
            pass
        meta = FieldMeta.parseField(key, valType)
        if key in d:
            raise Exception(f"duplicated key: {key}")
        d[key] = meta.format(val)
        pass
    return d


def intIf(val):
    if isinstance(val, float):
        if val == float(int(val)):
            return int(val)
    return val



# encoding: utf-8
"""Export data from Excel file.

Usage:
    excel2xx.py json    <excel> [options]
    excel2xx.py msgpack <excel> [options]
    excel2xx.py mako    <excel> --template=FILE [options]
    excel2xx.py --version

Arguments:
    <excel>                        Excel file path.

Options:
    -h --help                      show this help message and exit.
    -v --version                   show version and exit.
    -o --output=FILE               output to file.
    -r --row-number=ROW_NUMBER     first row. [default: 2]
    -v --verbose                   show debug infomation.
    -vv --verbose2                 show more debug infomation.
"""
from __future__ import unicode_literals, print_function
import os
import traceback
from docopt import docopt
from excel2xx import Excel, export, FieldMeta


def main(args):
    excelFile = args["<excel>"]
    outputFile = args["--output"]

    if excelFile:
        excelFile = os.path.realpath(excelFile)
    if outputFile:
        outputFile = os.path.realpath(outputFile)

    if not os.path.isfile(excelFile):
        print("Unexist file:" + excelFile)
        return 1

    excel = Excel(excelFile, fieldMeta=FieldMeta())

    if args["json"]:
        export.toJson(excel, args["--output"] or "%(excelFile)s.json" % locals())
    elif args["msgpack"]:
        export.toMsgPack(excel, args["--output"] or "%(excelFile)s.json" % locals())
    elif args["mako"]:
        export.toMako(
            excel, args["--output"] or "%(--template)s.mako" % args, args["--template"]
        )
    else:
        print("Invalid subcmd.")
        return 2

    return 0


def main_docopt():
    args = docopt(__doc__)
    if args["--verbose2"]:
        print(args)
    try:
        return main(args)
    except Exception as e:
        errorMsg = traceback.format_exc() if args["--verbose"] else e
        print(errorMsg)
        return 2
    pass


if __name__ == "__main__":
    exit(main_docopt())



from setuptools import setup
from io import open
import os

root_dir = os.path.dirname(__file__)

fpath = os.path.join(root_dir, "README.md")
readme = open(fpath, "r", encoding="utf-8").read()


setup(
    name="excel2xx",
    version="0.10.0",
    packages=["excel2xx"],
    url="https://github.com/cupen/excel2xx",
    license="WTFPL",
    author="cupen",
    author_email="xcupen@gmail.com",
    description="Extract data from excel file, and export to json, msgpack, or any code(mako template).",
    long_description=readme,
    long_description_content_type="text/markdown",
    install_requires=[
        "xlrd == 1.2.*",
        "docopt >= 0.6.0",
        "mako == 1.1.*",
        "msgpack-python >= 0.4.8",
        "colorama >= 0.4.6",
    ],
    entry_points={
        "console_scripts": [
            "excel2xx=excel2xx.main:main_docopt",
        ],
    },
    python_requires=">=3.6,!=3.6.1",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: Implementation :: CPython",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
